/*fastDummyprinter, Copyright (C) 2012 Laure Gonnord*/
/*This file was part of Aspic Distribution, (C) Laure Gonnord 2007*/
/**/
/*This library is free software; you can redistribute it and/or*/
/*modify it under the terms of the GNU Lesser General Public*/
/*License as published by the Free Software Foundation; either*/
/*version 2.1 of the License, or (at your option) any later version.*/
/**/
/*This library is distributed in the hope that it will be useful,*/
/*but WITHOUT ANY WARRANTY; without even the implied warranty of*/
/*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU*/
/*Lesser General Public License for more details.*/
/**/
/*You should have received a copy of the GNU Lesser General Public*/
/*License along with this library; if not, write to the Free Software*/
/*Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA*/
/*02110-1301 USA*/


%{
open Types
(* open Syntax *)
open Expressions

let stlist = ref []
let varlist = ref []
let paramlist = ref []

%}

/* %============================================================== */
/* \section{Tokens} */
/* %============================================================== */

%token TK_EOF

/* dlimiteurs */
%token TK_LPAREN TK_RPAREN TK_LBRACE TK_RBRACE TK_LBRACKET TK_RBRACKET
%token TK_COMMA TK_SEMI
%token TK_PRIME
%token TK_COLON
%token TK_VBAR TK_RARROW

/* oprations boolennes */
%token TK_TRUE TK_FALSE
%token TK_NOT TK_OR TK_AND TK_NOTEQUAL TK_EQUAL TK_SHARP TK_IMPLY TK_IN

/* oprations arithmtiques */
%token TK_MUL TK_DIV TK_PLUS TK_MINUS

/* oprations mixtes */
%token TK_GREATEREQUAL TK_LESSEQUAL TK_GREATER TK_LESS TK_ASSIGN

/* controle */
%token TK_IF TK_THEN TK_ELSE

/* dclarations */
%token TK_TYPEDEF TK_ENUM TK_BOOL TK_INT TK_REAL TK_UINT TK_SINT
%token TK_INVARIANT TK_BLOCKED TK_ANYTHING TK_INITIAL TK_SPACE TK_FINAL TK_ASSERTION
%token TK_INPUT
%token TK_LOCAL
%token TK_STATE
%token TK_CONST
%token TK_TRANS TK_DEFINITION
%token TK_CONTINUOUS
%token TK_CONTROL TK_AUTOMATON TK_LOCATION TK_EDGE TK_PRIORITY
%token <int> TK_NUM
%token <Rational.t> TK_RAT
%token <Symbol.t> TK_ID
%token <Symbol.t> TK_IDEXCLA


/* associativit */
%nonassoc TK_EQUIV
%right TK_IMPLY
%left TK_OR TK_OROR
%left TK_AND TK_ANDAND
%nonassoc TK_NOT TK_EXCLA
%nonassoc TK_GREATER TK_GREATEREQUAL TK_LESS TK_LESSEQUAL TK_EQUAL TK_NEQUAL
%left TK_PLUS TK_MINUS
%left TK_MUL TK_DIV

/*fast*/
%token TK_MODEL
%token TK_STRATEGY
%token TK_STRAT
%token TK_VAR
%token TK_STATES
%token TK_FROM
%token TK_TO
%token TK_GUARD
%token TK_ACTION
%token TK_REGION
%token TK_REGINIT
%token TK_REGBAD
%token TK_ANDAND
%token TK_OROR
%token TK_EXCLA
%token TK_PRE
%token TK_POST
%token TK_SETMAXSTATE
%token TK_SETMAXACC
%token TK_PRINT
%token TK_BOOLEAN
%token TK_STRATSUBSET 
%token TK_STRATEQSET
%token TK_STRATISEMPTY
%token TK_STRATTRANS
%token TK_EXISTS
%token TK_STRATTRUE
%token TK_STRATFALSE
%token TK_ENDIF
%token TK_EQUIV
%token TK_DOUBLEQUOTE
%token TK_INDET
%token TK_AT
%token TK_PARAM

%start prog

%type <Types.progfast> prog
%%


prog: model strat
{ 
  { 
    model = $1;
    strat = $2
  }
}

/*MODEL*/
model : TK_MODEL id TK_LBRACE vars param states transitions TK_RBRACE
{
  { 
    idmodel =$2;
    vars = List.rev $4;
    parameters = List.rev $5;
    states = List.rev $6;
    transitions = List.rev $7;
  }
}


vars : TK_VAR idlist TK_SEMI
{
  varlist:=$2;$2
}
| TK_VAR TK_SEMI {[]}



param : TK_AT TK_PARAM idlist TK_SEMI
{
  failwith "hopla";
  paramlist:=$3;$3
}
| {[]}

id:
TK_ID { $1 }

idexcla:
TK_IDEXCLA { $1 }

idlist:
id { [$1] }
| idlist TK_COMMA id { $3::$1 }
    

states : TK_STATES idlist TK_SEMI
{
  stlist:=$2;$2
}


transitions : translist 
{
  $1
}

translist :
trans { [$1] }
| translist trans {$2::$1}


trans : TK_TRANS id TK_ASSIGN TK_LBRACE intrans TK_RBRACE TK_SEMI
{
  let (fromid,toid,guard,action) = $5 in
    if (not(List.mem fromid !stlist))
	then failwith ("state "^(Symbol.to_string fromid)^" is not declared")
	else if (not(List.mem toid !stlist))
	then failwith ("state "^(Symbol.to_string toid)^" is not declared")
    else
    {
      ident = $2;
      fromid = fromid;
      toid = toid;
      guard = guard;
      action = action
    }
}


intrans : TK_FROM TK_ASSIGN id TK_SEMI TK_TO TK_ASSIGN id TK_SEMI TK_GUARD TK_ASSIGN guard TK_SEMI TK_ACTION TK_ASSIGN action TK_SEMI
{
  ($3,$7,$11,List.rev $15)
}

action : 
| {[]} /*action ne rien faire */
| affectation {[$1]}
| action TK_COMMA affectation {$1@[$3]}

affectation : 
|id TK_PRIME TK_EQUAL TK_INDET 
{ if not(List.mem $1 !varlist) then 
    failwith ("variable "^(Symbol.to_string $1)^" is not declared")
  else ($1,Indet($1))}
| id TK_PRIME TK_EQUAL numexpr
{ if not(List.mem $1 !varlist) then 
    failwith ("variable "^(Symbol.to_string $1)^" is not declared")
  else ($1,$4) }

/*Strategy*/

strat : TK_STRATEGY id TK_LBRACE instructionlist TK_RBRACE
{
  {
    idstrat = $2;
    instlist = $4;
  }
}


instructionlist : 
instruction {[$1]}
| instructionlist instruction {$2::$1} 


instruction : 
TK_REGION TK_REGINIT TK_ASSIGN region TK_SEMI{Declaration(Region(Initreg,$4))}
| TK_REGION TK_REGBAD TK_ASSIGN region TK_SEMI{Declaration(Region(Badreg,$4))}
| TK_REGION id TK_ASSIGN region TK_SEMI{Declaration(Region(Otherreg($2),$4))}
| TK_STRATTRANS id TK_ASSIGN  idlistbraq TK_SEMI {Declaration(Transitions($2,$4))}
| TK_BOOLEAN id TK_ASSIGN boolfastexpr TK_SEMI {Declaration(Boolean($2,$4))} /*ignore*/
| TK_SETMAXSTATE  TK_LPAREN integ TK_RPAREN TK_SEMI{Proc(1,$3)} 
| TK_SETMAXACC TK_LPAREN integ TK_RPAREN TK_SEMI {Proc(2,$3)} 
| TK_PRINT TK_LPAREN id  TK_RPAREN TK_SEMI{Print} /*ignore*/
| TK_PRINT TK_LPAREN chaine  TK_RPAREN TK_SEMI{Print} /*ignore*/
| TK_IF boolfastexpr TK_THEN instructionlist suite {Ite($2,$4,$5)} /*ignore*/

suite :
TK_ENDIF {[]}
| TK_ELSE instructionlist TK_ENDIF {$2}

chaine :
idexcla {}
| TK_DOUBLEQUOTE idandspace TK_DOUBLEQUOTE {}

idandspace : 
id {}
| id idandspace {}


idlistbraq :
id {[$1]}
| TK_LBRACE idlist TK_RBRACE{ $2 }



integ:
| TK_NUM {$1}

region : 
regionterm {$1}
| region TK_ANDAND region {Inter($1,$3)}
| region TK_OROR region {Union($1,$3)}
| TK_EXCLA region {Complement($2)}
| TK_LPAREN region TK_RPAREN {$2}
| TK_PRE TK_MUL TK_LPAREN region TK_COMMA  idlist TK_RPAREN {(PreStar($4,$6,0))}
| TK_PRE TK_MUL TK_LPAREN region TK_COMMA  idlist TK_COMMA  integ TK_RPAREN {PreStar($4,$6,$8)}
| TK_POST TK_MUL TK_LPAREN region TK_COMMA  idlist TK_RPAREN {(PostStar($4,$6,0))}
| TK_POST TK_MUL TK_LPAREN region TK_COMMA  idlist TK_COMMA integ TK_RPAREN {PostStar($4,$6,$8)}
		 
regionterm : 
id {RegiontermI($1)}
| TK_LBRACE guard TK_RBRACE {RegiontermF($2)}
| TK_REGINIT {let _ = Symbol.add "init" in RegiontermI(Symbol.of_string("init"))}

boolfastexpr :
| id {NoBFexp} 
| TK_TRUE {NoBFexp}
| TK_FALSE {NoBFexp}
| TK_STRATSUBSET TK_LPAREN region TK_COMMA region TK_RPAREN {NoBFexp}
| TK_STRATEQSET TK_LPAREN region TK_COMMA region TK_RPAREN {NoBFexp}
| TK_STRATISEMPTY  TK_LPAREN region TK_RPAREN{NoBFexp}
| boolfastexpr TK_AND boolfastexpr {NoBFexp}
| boolfastexpr TK_OR boolfastexpr {NoBFexp}
| TK_EXCLA boolfastexpr {NoBFexp}
| TK_LPAREN boolfastexpr TK_RPAREN {NoBFexp} 

/*guards are formula over numerical constraints*/
/*non primed formula*/

guard : 
 | TK_LPAREN guard TK_RPAREN {$2}
 | guard TK_ANDAND guard {OpBoolB(And,$1,$3)}
 | guard TK_OROR guard {OpBoolB(Or,$1,$3)}
 | guard TK_IMPLY guard {OpBoolB(Imply,$1,$3)}
 | TK_EXCLA  TK_LPAREN guard TK_RPAREN {Neg($3)}
 | term {Term($1)}

term:
 | TK_TRUE  {CBool(true)}
 | TK_FALSE {CBool(false)}
 | TK_STATE TK_EQUAL id {Stateis($3)}
 | exprcompare {$1}

exprcompare:
 | numexpr TK_GREATEREQUAL numexpr { ExprNum(SupEq,$1,$3) }
 | numexpr TK_GREATER numexpr { ExprNum(Sup,$1,$3) }
 | numexpr TK_LESSEQUAL numexpr { ExprNum(LessEq,$1,$3) }
 | numexpr TK_LESS numexpr { ExprNum(Less,$1,$3) }
 | numexpr TK_EQUAL numexpr { ExprNum(Eq,$1,$3) }
 | numexpr TK_NOTEQUAL numexpr { ExprNum(NotEq,$1,$3) }
     

numexpr :
expr6 {$1}

expr6:
expr7 { $1 }
| expr6 TK_PLUS expr7 { OpNum(Plus,$1,$3) }
| expr6 TK_MINUS expr7 { OpNum(Minus,$1,$3) }
expr7:
  expr8 { $1 }
| expr7 TK_MUL expr8 { OpNum(Times,$1,$3) }
| expr7 TK_DIV expr8 { OpNum(Div,$1,$3) }
expr8:
| numcst   { $1 }
| id { Ref($1) }
| numcst id { OpNum(Times,$1,Ref($2)) }
| TK_LPAREN expr6 TK_RPAREN { $2 }
| TK_MINUS expr8 { OpNum(Minus,CNat(0),$2) }
numcst:
  TK_NUM { CNat($1) }
| TK_RAT { CRat($1) }
