(*fastDummyprinter, Copyright (C) 2012 Laure Gonnord*)
(*This file was part of Aspic Distribution, (C) Laure Gonnord 2007*)
(*  *)
(* This library is free software; you can redistribute it and/or *)
(* modify it under the terms of the GNU Lesser General Public *)
(* License as published by the Free Software Foundation; either *)
(* version 2.1 of the License, or (at your option) any later version. *)
(*  *)
(* This library is distributed in the hope that it will be useful, *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *)
(* Lesser General Public License for more details. *)
(*  *)
(* You should have received a copy of the GNU Lesser General Public *)
(* License along with this library; if not, write to the Free Software *)
(* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA *)
(* 02110-1301 USA *)


open Format
open Expressions

(*types for the translation of fast to the internal structure*)
exception FastInputError of string

type transitionfast = {
  ident : Symbol.t;
  fromid : Symbol.t;
  toid : Symbol.t; 
  guard : Expressions.expr_bool;
  action : (Symbol.t *  Expressions.expr_num) list; (*left = id non primed, right = expression*)
}

let print_actionfast fmt act = 
  let debut = ref true in 
  List.iter (
    fun (id,act) ->
      if (!debut) then debut:=false else pp_print_string fmt ", ";
      fprintf fmt "%s'=" (Symbol.to_string id);
(*       print_symbexp fmt act; *)
      print_exprnum fmt act;
  ) act

let print_trans_fast (tr:transitionfast) fmt = 
  fprintf fmt " transition %s := {@."(Symbol.to_string tr.ident);
  fprintf fmt "      from  := %s ;@."(Symbol.to_string tr.fromid);
  fprintf fmt "      to    := %s ;@."(Symbol.to_string tr.toid);
  let theguard = tr.guard in 
    fprintf fmt "      guard := ";  print_exprbool fmt theguard;
    fprintf fmt ";@.      action := ";  print_actionfast fmt tr.action;
  fprintf fmt ";@.  };@.@."

type modelfast = {
  idmodel : Symbol.t;
  mutable vars : Symbol.t list ;
  mutable parameters : Symbol.t list ; (*sous ens de vars*)
  mutable states : Symbol.t list ;
  mutable transitions : transitionfast list ; 
}


type constantboolfast = Id of Symbol.t  | Cst_true | Cst_false


type regionfast = 
  | RegiontermI of Symbol.t (*ident*)
  | RegiontermF of   Expressions.expr_bool (*non primed formula*)
  | PreStar of regionfast * (Symbol.t list) * int 
(*the int reprents the >=size of accelerated cycles*)
  | PostStar of regionfast * (Symbol.t list) * int
  | Union of regionfast * regionfast 
  | Inter of regionfast * regionfast
  | Complement of regionfast

type boolfastexpr = 
  | Cstfast of constantboolfast
  | SpecialEqset of regionfast * regionfast
  | SpecialSubset of regionfast * regionfast
  | SpecialIsEmpty of regionfast
  | Negation of boolfastexpr
  | BoolAnd of boolfastexpr * boolfastexpr
  | BoolOr of boolfastexpr * boolfastexpr
  | BoolEq of boolfastexpr * boolfastexpr
  | NoBFexp
type typeregion = Initreg| Badreg | Otherreg of  Symbol.t

type declfast = 
  | Region of typeregion * regionfast (*init/bad/other, regiondef *)
  | Transitions of Symbol.t  * Symbol.t list (*ident, symbol list of trans*)
  | Boolean of Symbol.t * boolfastexpr  (*ident, formula*)

type fastinstr = 
  |  Declaration of declfast
  |  Ite of (boolfastexpr * fastinstr list * fastinstr list)
  |  Proc of int*int
  |  Print 
(*procedure : type 1 = MaxSate, type 2 = MaxAcc, print = ignore *)

type strategyfast = {
  idstrat : Symbol.t;
  instlist : fastinstr list;
}

type progfast = {
  model : modelfast ; 
  strat : strategyfast;
}

let print_symblist (els:Symbol.t list)  fmt sepsymbol =
  let debut = ref true in
  List.iter ( fun el -> 
    if (!debut) then debut:=false else pp_print_string fmt sepsymbol;
    pp_print_string fmt (Symbol.to_string el)
  ) els

(*not complete*)
let print_strategy_list (strat : strategyfast) (bfinal:bool) fmt = 
(*just printing initial and bad state*)
  let instlist = strat.instlist in
    List.iter ( 
      fun el2-> 
	match el2 with 
	  | Declaration(el) -> (
	      match el with
		| Region(Badreg,reg) ->
		    ( if bfinal then match reg with 
			| RegiontermF(rexpreg) ->
			    begin
			      fprintf fmt "Region bad := { ";
(* 			      print_symbexp fmt rexpreg; *)
			      print_exprbool fmt rexpreg;
			      fprintf fmt "};@.@.";
			    end
			| _ -> ();
		    )
		| Region(Initreg,reg) -> 
		    ( match reg with 
			| RegiontermF(rexpreg) ->
			    begin
			      fprintf fmt "Region init := { ";
(* 			      print_symbexp fmt rexpreg; *)
			      print_exprbool fmt rexpreg;
			      fprintf fmt "};@.@.";
			    end
			| _ -> ();
		    )
		| _ -> ()
	    )
	  | _ -> ()
    ) instlist



let print_fast (prog:progfast) comments lparam bfinal fmt = 
  let model = prog.model and strat = prog.strat in
    (*print comments*)
    fprintf fmt "// %s@." comments;
    (*print the model*)
    fprintf fmt "model %s { @." (Symbol.to_string model.idmodel);
    fprintf fmt "@.//parameters ";
    print_symblist lparam fmt ", ";
    fprintf fmt ";@.@. var ";
    print_symblist model.vars fmt ", ";
    fprintf fmt ";@.@. states ";
    print_symblist model.states fmt ", ";
    fprintf fmt ";@.@.";
    List.iter (fun trans -> 
		 print_trans_fast trans fmt) 
      model.transitions;
    fprintf fmt "} @.@.";
    (*now the stategy*)
    fprintf fmt "strategy %s { @. @." (Symbol.to_string strat.idstrat);
    print_strategy_list strat bfinal fmt ; 
    fprintf fmt "} @."

