(*fastDummyprinter, Copyright (C) 2012 Laure Gonnord*)
(*This file was part of Aspic Distribution, (C) Laure Gonnord 2007*)
(*  *)
(* This library is free software; you can redistribute it and/or *)
(* modify it under the terms of the GNU Lesser General Public *)
(* License as published by the Free Software Foundation; either *)
(* version 2.1 of the License, or (at your option) any later version. *)
(*  *)
(* This library is distributed in the hope that it will be useful, *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *)
(* Lesser General Public License for more details. *)
(*  *)
(* You should have received a copy of the GNU Lesser General Public *)
(* License along with this library; if not, write to the Free Software *)
(* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA *)
(* 02110-1301 USA *)


(*types for the translation of fast to the internal structure*)

exception FastInputError of string

type transitionfast = {
  ident : Symbol.t;
  fromid : Symbol.t;
  toid : Symbol.t; 
  guard : Expressions.expr_bool;
  action : (Symbol.t * Expressions.expr_num ) list; (*left = id non primed, right = expression*)
}

type modelfast = {
  idmodel : Symbol.t ; 
  mutable vars : Symbol.t list ;
  mutable parameters : Symbol.t list ; (*sous ens de vars*)
  mutable states : Symbol.t list ;
  mutable transitions : transitionfast list ; 
}

type constantboolfast = Id of Symbol.t  | Cst_true | Cst_false


type regionfast = 
  | RegiontermI of Symbol.t (*ident*)
  | RegiontermF of  Expressions.expr_bool
  | PreStar of regionfast * (Symbol.t list) * int 
(*the int reprents the >=size of accelerated cycles*)
  | PostStar of regionfast * (Symbol.t list) * int
  | Union of regionfast * regionfast 
  | Inter of regionfast * regionfast
  | Complement of regionfast

(*   | Stateis of Symbol.t ?? *)

type boolfastexpr = 
  | Cstfast of constantboolfast
  | SpecialEqset of regionfast * regionfast
  | SpecialSubset of regionfast * regionfast
  | SpecialIsEmpty of regionfast
  | Negation of boolfastexpr
  | BoolAnd of boolfastexpr * boolfastexpr
  | BoolOr of boolfastexpr * boolfastexpr
  | BoolEq of boolfastexpr* boolfastexpr
  | NoBFexp

type typeregion = Initreg| Badreg | Otherreg of Symbol.t

type declfast = 
  | Region of typeregion *  regionfast (*init/bad/other, ident,region*)
  | Transitions of Symbol.t  * Symbol.t list (*ident, symbol list of trans*)
  | Boolean of Symbol.t * boolfastexpr  (*ident, formula , not used*)

type fastinstr = 
  |  Declaration of declfast
  |  Ite of (boolfastexpr * fastinstr list * fastinstr list)
  |  Proc of int*int
(*procedure : type 1 = MaxSate, type 2 = MaxAcc, print = ignore*)
  |  Print
       
type strategyfast = {
  idstrat : Symbol.t;
  instlist : fastinstr list;
}

type progfast = {
  model : modelfast ; 
  strat : strategyfast;
}

val print_symblist : Symbol.t list -> Format.formatter -> string  -> unit
val print_actionfast : Format.formatter -> (Symbol.t *Expressions.expr_num ) list -> unit
val print_fast : progfast -> string ->Symbol.t list -> bool ->  Format.formatter ->unit
